@@ -1,5 +1,7 @@
 //! Signal processing using PGN/SPN definitions.
 
+use encoding::{CodecError, DecoderTrap, Encoding};
+use encoding::all::ISO_8859_1;
 use regex::Regex;
 use std::collections::HashMap;
 use std::error::Error;
@@ -45,7 +47,7 @@ impl PgnLibrary {
     }
 
     /// Convenience function for loading an entire DBC file into a returned `PgnLibrary`.  This
-    /// function ignores unparsable lines as well as `Entry` variants which don't apply to
+    /// function ignores unparseable lines as well as `Entry` variants which don't apply to
     /// `PgnLibrary` (such as `Entry::Version`).  Fails on `io::Error`.
     ///
     /// # Example
@@ -56,16 +58,57 @@ impl PgnLibrary {
     /// let lib: PgnLibrary = PgnLibrary::from_dbc_file("./tests/data/sample.dbc").unwrap();
     ///
     /// ```
-    pub fn from_dbc_file<P: AsRef<Path>>(path: P) -> io::Result<Self> {
+    pub fn from_dbc_file<P>(path: P) -> io::Result<Self>
+    where
+        P: AsRef<Path>,
+    {
+        Self::from_encoded_dbc_file(path, ISO_8859_1)
+    }
+
+    /// Convenience function for loading an entire DBC file into a returned `PgnLibrary`, using
+    /// a specified `Encoding` codec. This function ignores unparseable lines as well as `Entry`
+    /// variants which don't apply to `PgnLibrary` (such as `Entry::Version`).
+    ///
+    /// This function is currently considered unstable and subject to change or removal.
+    ///
+    /// # Example
+    ///
+    /// ```rust
+    /// extern crate canparse;
+    /// extern crate encoding;
+    ///
+    /// use canparse::pgn::PgnLibrary;
+    /// use encoding::Encoding;
+    /// use encoding::all::ISO_8859_1;
+    ///
+    /// let lib: PgnLibrary = PgnLibrary::from_encoded_dbc_file("./tests/data/sample.dbc",
+    ///                                                         ISO_8859_1).unwrap();
+    ///
+    /// ```
+    #[doc(hidden)]
+    pub fn from_encoded_dbc_file<P, E>(path: P, encoding: &E) -> io::Result<Self>
+    where
+        P: AsRef<Path>,
+        E: Encoding,
+    {
         let mut lib = PgnLibrary::default();
 
-        let f = File::open(path)?;
-        let f = BufReader::new(f);
+        let data = File::open(path)
+            .and_then(|mut f| {
+                let mut contents: Vec<u8> = Vec::new();
+                f.read_to_end(&mut contents)
+                    .map(|_bytes_read| contents)
+            })
+            .and_then(|mut contents| {
+                encoding.decode(contents.as_slice(), DecoderTrap::Replace)
+                    .map_err(|e| io::Error::new(io::ErrorKind::Other, e))
+            })?;
 
-        for l in f.lines() {
-            let line = l?;
-            if let Ok(entry) = Entry::from_str(line.as_str()) {
-                lib.add_entry(entry).ok();
+        for (idx, line) in data.lines().enumerate() {
+            if let Ok(entry) = Entry::from_str(line) {
+                if let Err(_e) = lib.add_entry(entry) {
+                    // TODO: Handle add_entry error
+                }
             }
         }
 
@@ -87,11 +130,13 @@ impl PgnLibrary {
     ///
     /// let mut lib = PgnLibrary::new( HashMap::default() );
     ///
-    /// let br = include_bytes!("../tests/data/sample.dbc");
+    /// // File is ISO-8859-1 and needs to be converted before iterating
+    /// // with `lines`. `PgnLibrary::from_dbc_file` does this for you.
+    /// let data: String = include_bytes!("../tests/data/sample.dbc")
+    ///     .iter().map(|b| *b as char).collect();
     ///
-    /// for l in br.lines() {
-    ///     let line = l.unwrap();
-    ///     if let Some(entry) = Entry::from_str(line.as_str()).ok() {
+    /// for line in data.lines() {
+    ///     if let Some(entry) = Entry::from_str(line).ok() {
     ///         lib.add_entry(entry).ok();
     ///     }
     /// }
@@ -132,7 +177,7 @@ impl PgnLibrary {
     }
 
     /// Returns a `SpnDefinition` entry reference, if it exists.
-    pub fn get_spn(&self, name: &String) -> Option<&SpnDefinition> {
+    pub fn get_spn(&self, name: &str) -> Option<&SpnDefinition> {
         self.pgns.iter().filter_map(|pgn| {
             pgn.1.spns.get(name)
         }).next()
@@ -383,28 +428,19 @@ fn parse_array(bit_len: usize, start_bit: usize, little_endian: bool, scale: f32
 /// the real calculations happen.
 fn parse_message(bit_len: usize, start_bit: usize, little_endian: bool, scale: f32, offset: f32, msg: &[u8]) -> Option<f32> {
 
-    let num_bytes: usize = f32::ceil( (bit_len as f32)/8.0 ) as usize;
-    let byte_pos: usize = f32::floor( (start_bit as f32)/8.0 ) as usize;
-    let mut val32: u32 = 0;
-
-    // TODO: There has to be a clean way to parameterize iter transforms
-    if little_endian {
-        for (i, n) in msg.iter().skip(byte_pos).take(num_bytes).enumerate() {
-            val32 += ((n & 0xFF) as u32) * (SHIFT_BYTE_LOOKUP[i] as u32);
-        }
-    } else {
-        for (i, n) in msg.iter().rev().skip(byte_pos).take(num_bytes).enumerate() {
-            val32 += ((n & 0xFF) as u32) * (SHIFT_BYTE_LOOKUP[i] as u32);
-        }
+    if msg.len() < 8 {
+        return None
     }
+    let msg64: u64 = match little_endian {
+        true => LittleEndian::read_u64(msg),
+        false => BigEndian::read_u64(msg)
+    };
 
-    let bit_pos = start_bit % 8;
-    val32 >>= bit_pos;
+    let bit_mask: u64 = 2u64.pow(bit_len as u32) - 1;
 
-    let bit_mask = 2u32.pow(bit_len as u32) - 1;
-    val32 &= bit_mask;
-
-    Some(val32 as f32 * scale + offset)
+    Some(
+        ( ( ( msg64 >> start_bit ) & bit_mask ) as f32 ) * scale + offset
+    )
 }
 
 /// The collection of functions for parsing CAN messages `N` into their defined signal values.
@@ -626,7 +662,7 @@ mod tests {
         assert_eq!(
             *PGNLIB_ONE.get_pgn(0xF004)
                 .expect("failed to get PgnDefinition from PgnLibrary")
-                .spns.get(&"Engine_Speed".to_string())
+                .spns.get("Engine_Speed")
                 .expect("failed to get SpnDefinition from PgnDefinition"),
             *SPNDEF
         );
@@ -644,12 +680,12 @@ mod tests {
     #[test]
     fn test_parse_array() {
         assert_eq!(
-        SPNDEF.parse_message(&MSG as &[u8; 8]).unwrap(),
-        2728.5
+            SPNDEF.parse_message(&MSG as &[u8; 8]).unwrap(),
+            2728.5
         );
         assert_eq!(
-        SPNDEF_BE.parse_message(&MSG_BE as &[u8; 8]).unwrap(),
-        2728.5
+            SPNDEF_BE.parse_message(&MSG_BE as &[u8; 8]).unwrap(),
+            2728.5
         );
     }
 
@@ -659,6 +695,12 @@ mod tests {
             SPNDEF.parse_message(&MSG[..]).unwrap(),
             2728.5
         );
+        assert_eq!(
+            SPNDEF_BE.parse_message(&MSG_BE[..]).unwrap(),
+            2728.5
+        );
+        assert!(SPNDEF.parse_message(&MSG[..7]).is_none());
+        assert!(SPNDEF_BE.parse_message(&MSG_BE[..7]).is_none());
     }
 
     #[test]
@@ -667,6 +709,10 @@ mod tests {
             SPNDEF.parser()(&MSG[..]).unwrap(),
             2728.5
         );
+        assert_eq!(
+            SPNDEF_BE.parser()(&MSG_BE[..]).unwrap(),
+            2728.5
+        );
     }
 
 
@@ -685,6 +731,12 @@ mod tests {
             false,
             false
         ).unwrap();
+        static ref FRAME_BE: CANFrame = CANFrame::new(
+            0,
+            &MSG_BE[..],
+            false,
+            false
+        ).unwrap();
         }
 
         #[test]
@@ -693,19 +745,20 @@ mod tests {
                 SPNDEF.parser()(&FRAME as &CANFrame).unwrap(),
                 2728.5
             );
+            assert_eq!(
+                SPNDEF_BE.parser()(&FRAME_BE as &CANFrame).unwrap(),
+                2728.5
+            );
         }
 
         #[test]
         fn test_parse_canframe() {
-            let frame = CANFrame::new(
-                0,
-                &MSG[..],
-                false,
-                false
-            ).unwrap();
-
             assert_eq!(
-                SPNDEF.parse_message(&frame).unwrap(),
+                SPNDEF.parse_message(&FRAME as &CANFrame).unwrap(),
+                2728.5
+            );
+            assert_eq!(
+                SPNDEF_BE.parse_message(&FRAME_BE as &CANFrame).unwrap(),
                 2728.5
             );
         }
