@@ -5,25 +5,24 @@ use canparse::dbc::Entry;
 use canparse::pgn::{PgnLibrary, SpnDefinition, ParseMessage};
 use std::str::FromStr;
 use std::collections::HashMap;
-use std::io::BufRead;
 
 #[test]
 fn pgnlib_build_parse() {
     let mut lib = PgnLibrary::new( HashMap::default() );
 
-    let br = include_bytes!("./data/sample.dbc");
+    let data: String = include_bytes!("./data/sample.dbc")
+        .iter().map(|b| *b as char).collect();
 
     // Parse db lines into PgnLibrary
-    for l in br.lines() {
-        let line = l.unwrap();
-        if let Ok(entry) = Entry::from_str(line.as_str()) {
+    for line in data.lines() {
+        if let Ok(entry) = Entry::from_str(&line) {
             lib.add_entry(entry).ok();
         }
     }
 
     // Pull signal definition for engine speed
     let enginespeed_def: &SpnDefinition = lib
-        .get_spn(&"Engine_Speed".to_string()).unwrap();
+        .get_spn("Engine_Speed").unwrap();
 
     // Parse frame containing engine speed
     let msg: [u8; 8] = [0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88];
